define([
	"dojo/_base/declare",
	"dojo/when",

	"aps/_View",
	"aps/ResourceStore",

	"./displayError.js"
], function(
	declare,
	when,

	_View,
	Store,

	displayError
) {
	var page, grid;

	return declare(_View, {
		init: function() {
			/* Create the data store */
			var store = new Store({
				apsType: "http://aps-standard.org/samples/sample-offering/vps/1.0",
				target: "/aps/2/resources/"
			});

			/* Create a handler for the *New* button click */
			var add = function() {
				/* Start the process of creating a VPS by going to the first screen */
				aps.apsc.gotoView("vps-wizard");
			};

			var refresh = function() {
				grid.refresh();
			};

			function changeState(state, btn) {
				var sel = grid.get("selectionArray");
				var counter = sel.length;

				/* Clear the current messages on the screen */
				page.get("messageList").removeAll();

				sel.forEach(function(vpsId) {
					console.log("I'm trying to change state of VPS with id = [" + vpsId + "]");

					var vps = {
						aps: {
							id: vpsId
						},
						state: state
					};

					/* Save the VPS properties */
					when(store.put(vps),
						/* If success, process the next VPS until the list is empty */
						function() {
							console.log("State of VPS with id = [" + vpsId + "] changed");
							sel.splice(sel.indexOf(vpsId), 1);
							grid.refresh();
							if (--counter === 0) {
								btn.cancel();
							} /* Remove busy state for button */
						},
						/* If failure, call the error handler */
						function(e) {
							displayError(e);
							if (--counter === 0) {
								btn.cancel();
							}
						}
					);
				});
			}

			/* Create the on-click handler for the *Stop* button */
			var stop = function() {
				changeState("Stopped", this);
			};

			/* Create the on-click handler for the *Start* button */
			var start = function() {
				changeState("Running", this);
			};

			/* Create the on-click handler for the *Delete* button */
			var remove = function() {
				var self = this;
				/* Get confirmation from the user for the delete operation */
				if (!confirm("Are you sure you want to delete VPSes?")) {
					self.cancel();
					return;
				}
				var sel = grid.get("selectionArray"),
					counter = sel.length;

				/* Clear the current messages on the screen */
				page.get("messageList").removeAll();

				sel.forEach(function(vpsId) {
					console.log("I'm trying to delete VPS with id = [" + vpsId + "]");

					/* Remove the VPS from the APS controller DB */
					when(store.remove(vpsId),
						/* If success, process the next VPS until the list is empty */
						function() {
							console.log("VPS with id = [" + vpsId + "] removed");
							sel.splice(sel.indexOf(vpsId), 1);
							grid.refresh();
							if (--counter === 0) {
								self.cancel();
							}
						},
						/* If failure, call the error handler */
						function(e) {
							displayError(e);
							if (--counter === 0) {
								self.cancel();
							}
						}
					);
				});
			};

			/* Define and return widgets */
			return ["aps/Grid", {
				id: this.genId("srv_grid"),
				store: store,
				selectionMode: "multiple",
				apsResourceViewId: "server-edit",
				columns: [{
					field: "name",
					name: "Name",
					type: "resourceName"
				}, {
					field: "hardware.memory",
					name: "RAM",
					title: "RAM"
				}, {
					field: "hardware.diskspace",
					name: "Disk Space",
					title: "Disk"
				}, {
					field: "hardware.CPU",
					name: "CPU"
				}, {
					field: "platform.OS.name",
					name: "OS"
				}, {
					field: "state",
					name: "State"
				}, {
					field: "userName",
					name: "VPS Owner"
				}, {
					field: "aps.status",
					name: "APS Status"
				}]
			}, [
				["aps/Toolbar", [
					["aps/ToolbarButton", {
						id: this.genId("srv_new"),
						iconClass: "fa-plus",
						type: "primary",
						label: "New",
						size: 3,
						onClick: add
					}],
					["aps/ToolbarButton", {
						id: this.genId("srv_start"),
						iconClass: "fa-play",
						type: "success",
						label: "Start",
						requireItems: true,
						size: 3,
						onClick: start
					}],
					["aps/ToolbarButton", {
						id: this.genId("srv_stop"),
						iconClass: "fa-pause",
						type: "warning",
						label: "Stop",
						requireItems: true,
						size: 3,
						onClick: stop
					}],
					["aps/ToolbarButton", {
						id: this.genId("srv_delete"),
						iconClass: "fa-trash",
						type: "danger",
						label: "Delete",
						requireItems: true,
						size: 3,
						onClick: remove
					}],
					["aps/ToolbarButton", {
						id: this.genId("srv_refresh"),
						iconClass: "fa-refresh",
						label: "Refresh",
						autoBusy: false,
						size: 3,
						onClick: refresh
					}]
				]]
			]];
		}, // End of Init
		onContext: function() {
			grid = grid || this.byId("srv_grid");
			page = page || this.byId("apsPageContainer");
			grid.refresh();
			aps.apsc.hideLoading();
		},
		onHide: function() {
			this.byId("srv_new").cancel();
		}
	}); // End of Declare
}); // End of Define